@@ -1,17 +1,8 @@
 <?php
 /**
- * VFS implementation for a standard filesystem.
+ * VFS implementation for a filesystem.
  *
- * Required parameters:
- * <pre>
- * 'vfsroot' - (string) The root path.
- * </pre>
- *
- * Note: The user that your webserver runs as (commonly 'nobody',
- * 'apache', or 'www-data') MUST have read/write permission to the
- * directory you specify as the 'vfsroot'.
- *
- * Copyright 2002-2011 Horde LLC (http://www.horde.org/)
+ * Copyright 2002-2013 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
@@ -44,10 +35,23 @@ class Horde_Vfs_File extends Horde_Vfs_Base
         )
     );
 
+    /**
+     * List of features that the VFS driver supports.
+     *
+     * @var array
+     */
+    protected $_features = array(
+        'readByteRange' => true,
+    );
+
     /**
      * Constructs a new Filesystem based VFS object.
      *
-     * @param array $params  A hash containing connection parameters.
+     * @param array $params  A hash containing connection parameters. REQUIRED
+     *                       parameters:
+     *   - vfsroot: (string) The root path.
+     *              Note: The user that your webserver runs as MUST have
+     *              read/write permission to this directory.
      */
     public function __construct($params = array())
     {
@@ -58,6 +62,8 @@ class Horde_Vfs_File extends Horde_Vfs_Base
              (substr($this->_params['vfsroot'], -1) == '\\'))) {
             $this->_params['vfsroot'] = substr($this->_params['vfsroot'], 0, strlen($this->_params['vfsroot']) - 1);
         }
+
+        $this->_connect();
     }
 
     /**
@@ -72,7 +78,7 @@ class Horde_Vfs_File extends Horde_Vfs_Base
     public function size($path, $name)
     {
         if (($size = @filesize($this->_getNativePath($path, $name))) === false) {
-            throw new Horde_Vfs_Exception(sprintf('Unable to check file size of "%s/%s".'), $path, $name);
+            throw new Horde_Vfs_Exception(sprintf('Unable to check file size of "%s/%s".', $path, $name));
         }
 
         return $size;
@@ -124,12 +130,10 @@ class Horde_Vfs_File extends Horde_Vfs_Base
      */
     public function readStream($path, $name)
     {
-        $mode = OS_WINDOWS ? 'rb' : 'r';
-        $stream = @fopen($this->_getNativePath($path, $name), $mode);
+        $stream = @fopen($this->_getNativePath($path, $name), 'rb');
         if (!is_resource($stream)) {
             throw new Horde_Vfs_Exception('Unable to open VFS file.');
         }
-
         return $stream;
     }
 
@@ -153,8 +157,7 @@ class Horde_Vfs_File extends Horde_Vfs_Base
      * @return string  The file data.
      * @throws Horde_Vfs_Exception
      */
-    public function readByteRange($path, $name, &$offset, $length = -1,
-                                  &$remaining)
+    public function readByteRange($path, $name, &$offset, $length, &$remaining)
     {
         if ($offset < 0) {
             throw new Horde_Vfs_Exception(sprintf('Wrong offset %d while reading a VFS file.', $offset));
@@ -197,7 +200,7 @@ class Horde_Vfs_File extends Horde_Vfs_Base
      *
      * @throws Horde_Vfs_Exception
      */
-    public function write($path, $name, $tmpFile, $autocreate = true)
+    public function write($path, $name, $tmpFile, $autocreate = false)
     {
         if (!@is_dir($this->_getNativePath($path))) {
             if ($autocreate) {
@@ -262,10 +265,7 @@ class Horde_Vfs_File extends Horde_Vfs_Base
      */
     public function copy($path, $name, $dest, $autocreate = false)
     {
-        $orig = $this->_getNativePath($path, $name);
-        if (preg_match('|^' . preg_quote($orig) . '/?$|', $dest)) {
-            throw new Horde_Vfs_Exception('Cannot copy file(s) - source and destination are the same.');
-        }
+        $this->_checkDestination($path, $dest);
 
         if ($autocreate) {
             $this->autocreatePath($dest);
@@ -277,6 +277,7 @@ class Horde_Vfs_File extends Horde_Vfs_Base
             }
         }
 
+        $orig = $this->_getNativePath($path, $name);
         $this->_checkQuotaWrite('file', $orig);
 
         if (!@copy($orig, $this->_getNativePath($dest, $name))) {
@@ -294,13 +295,13 @@ class Horde_Vfs_File extends Horde_Vfs_Base
      *
      * @throws Horde_Vfs_Exception
      */
-    public function writeData($path, $name, $data, $autocreate = true)
+    public function writeData($path, $name, $data, $autocreate = false)
     {
         if (!@is_dir($this->_getNativePath($path))) {
             if ($autocreate) {
                 $this->autocreatePath($path);
             } else {
-                throw new Horde_Vfs_Exception('VFS directory does not exist.');
+                throw new Horde_Vfs_Exception(sprintf('VFS directory %s does not exist.', $path));
             }
         }
 
@@ -408,12 +409,13 @@ class Horde_Vfs_File extends Horde_Vfs_Base
     }
 
     /**
-     * Return a list of the contents of a folder.
+     * Returns an unsorted file list of the specified directory.
      *
-     * @param string $path       The path of the directory.
-     * @param mixed $filter      String/hash to filter file/dirname on.
-     * @param boolean $dotfiles  Show dotfiles?
-     * @param boolean $dironly   Show only directories?
+     * @param string $path          The path of the directory.
+     * @param string|array $filter  Regular expression(s) to filter
+     *                              file/directory name on.
+     * @param boolean $dotfiles     Show dotfiles?
+     * @param boolean $dironly      Show only directories?
      *
      * @return array  File list.
      * @throws Horde_Vfs_Exception
@@ -459,7 +461,7 @@ class Horde_Vfs_File extends Horde_Vfs_Base
 
             // Group
             $file['group'] = filegroup($entry);
-            if (function_exists('posix_getgrgid') && (PHP_VERSION != '5.2.1')) {
+            if (function_exists('posix_getgrgid')) {
                 $group = posix_getgrgid($file['group']);
                 $file['group'] = $group['name'];
             }
@@ -537,42 +539,6 @@ class Horde_Vfs_File extends Horde_Vfs_Base
         return $files;
     }
 
-    /**
-     * Returns a sorted list of folders in specified directory.
-     *
-     * @param string $path         The path of the directory to get the
-     *                             directory list for.
-     * @param mixed $filter        Hash of items to filter based on folderlist.
-     * @param boolean $dotfolders  Include dotfolders?
-     *
-     * @return array  Folder list.
-     * @throws Horde_Vfs_Exception
-     */
-    function listFolders($path = '', $filter = null, $dotfolders = true)
-    {
-        $this->_connect();
-
-        $folders = array();
-        $folders[dirname($path)] = array(
-            'val' => dirname($path),
-            'abbrev' => '..',
-            'label' => '..'
-        );
-
-        $folderList = $this->listFolder($path, null, $dotfolders, true);
-        foreach ($folderList as $name => $files) {
-            $folders[$name] = array(
-                'val' => $path . '/' . $files['name'],
-                'abbrev' => $files['name'],
-                'label' => $path . '/' . $files['name']
-            );
-        }
-
-        ksort($folders);
-
-        return $folders;
-    }
-
     /**
      * Return Unix style perms.
      *
@@ -673,9 +639,9 @@ class Horde_Vfs_File extends Horde_Vfs_Base
 
             $path = str_replace('..', '', $path);
             if (substr($path, 0, 1) == '/') {
-                return $this->_params['vfsroot'] . $path . $name;
+                return $this->_params['vfsroot'] . rtrim($path, '/') . $name;
             } else {
-                return $this->_params['vfsroot'] . '/' . $path . $name;
+                return $this->_params['vfsroot'] . '/' . rtrim($path, '/') . $name;
             }
         }
 
